# coding: utf-8
from __future__ import unicode_literals

DB_HOST = '127.0.0.1'
DB_PORT = 5432
DB_NAME = 'postgres'
DB_USER = 'postgres'
DB_PASSWORD = 'postgres'

TELEGRAM_TOKEN = ''
TELEGRAM_CHATS = []

TIMEZONE = 'Europe/Moscow'
